@@ -28,7 +28,7 @@ from warnings import warn
 from . import _qpdf
 from ._qpdf import Object, ObjectType
 
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     from pikepdf import Pdf
 
 # By default pikepdf.Object will identify itself as pikepdf._qpdf.Object
@@ -58,7 +58,7 @@ class _NameObjectMeta(_ObjectMeta):
 
     def __getattr__(self, attr):
         if attr.startswith('_'):
-            return _ObjectMeta.__getattr__(attr)
+            return getattr(_ObjectMeta, attr)
         return Name('/' + attr)
 
     def __setattr__(self, attr, value):
@@ -169,6 +169,8 @@ class Array(Object, metaclass=_ObjectMeta):
             raise TypeError('Strings cannot be converted to arrays of chars')
         if a is None:
             a = []
+        if isinstance(a, Array):
+            return a.__copy__()
         return _qpdf._new_array(a)
 
 
@@ -198,11 +200,14 @@ class Dictionary(Object, metaclass=_ObjectMeta):
             pikepdf.Object
         """
         if kwargs and d is not None:
-            raise ValueError('Unsupported parameters')
+            raise ValueError('Cannot use both a mapping object and keyword args')
         if kwargs:
             # Add leading slash
             # Allows Dictionary(MediaBox=(0,0,1,1), Type=Name('/Page')...
             return _qpdf._new_dictionary({('/' + k): v for k, v in kwargs.items()})
+        if isinstance(d, Dictionary):
+            # Already a dictionary
+            return d.__copy__()
         if not d:
             d = {}
         if d and any(not key.startswith('/') for key in d.keys()):
@@ -217,19 +222,49 @@ class Stream(Object, metaclass=_ObjectMeta):
 
     def __new__(cls, owner: 'Pdf', data: bytes = None, d=None, **kwargs):
         """
+        Create a new stream object, which stores arbitrary binary data and may
+        or may not be compressed.
+
+        A stream dictionary is like a pikepdf.Dictionary or Python dict, except
+        it has a binary payload of data attached. The dictionary describes
+        how the data is compressed or encoded.
+
+        The dictionary may be initialized just like pikepdf.Dictionary is initialized,
+        using a mapping object or keyword arguments.
+
         Args:
             owner: The Pdf to which this stream shall be attached.
-            obj: The data bytes for the stream.
-            d: A mapping object that will be used to construct a ``Dictionary``.
-            kwargs: Keyword arguments that will define the dictionary. Do not set
-                /Filter or /Length here as pikepdf will manage these.
+            data: The data bytes for the stream.
+            d: An optional mapping object that will be used to construct the stream's
+                dictionary.
+            kwargs: Keyword arguments that will define the stream dictionary. Do not set
+                /Length here as pikepdf will manage this value. Set /Filter
+                if the data is already encoded in some format.
+            obj: Deprecated alias for *data*.
         Returns:
             pikepdf.Object
+
+        Examples:
+            Using kwargs:
+                >>> s1 = pikepdf.Stream(
+                        pdf,
+                        b"uncompressed image data",
+                        BitsPerComponent=8,
+                        ColorSpace=Name.DeviceRGB,
+                        ...
+                    )
+            Using dict:
+                >>> d = pikepdf.Dictionary(...)
+                >>> s2 = pikepdf.Stream(
+                        pdf,
+                        b"data",
+                        d
+                    )
         """
 
         # Support __new__(...obj=bytes) which should have been data=bytes,
         # drop in pikepdf 3
-        if 'obj' in kwargs:
+        if 'obj' in kwargs:  # pragma: no cover
             warn("Deprecated parameter 'obj', use 'data' instead", DeprecationWarning)
             if data is None:
                 data = kwargs['obj']
@@ -238,8 +273,11 @@ class Stream(Object, metaclass=_ObjectMeta):
         if data is None:
             raise TypeError("Must make Stream from binary data")
 
-        stream = _qpdf._new_stream(owner, data)
+        stream_dict = None
         if d or kwargs:
             stream_dict = Dictionary(d, **kwargs)
+
+        stream = _qpdf._new_stream(owner, data)
+        if stream_dict:
             stream.stream_dict = stream_dict
         return stream
